/*
 * $Id: libgnac-output.h 375 2009-07-17 07:08:04Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef __LIBGNAC_OUTPUT_H__
#define __LIBGNAC_OUTPUT_H__

#include <gio/gio.h>
#include <glib.h>

#include "libgnac-converter.h"

G_BEGIN_DECLS

#define RENAME_PATTERN_SEPARATOR '%'

#define RENAME_PATTERN_ARTIST       'a'
#define RENAME_PATTERN_ALBUM        'b'
#define RENAME_PATTERN_COMMENT      'c'
#define RENAME_PATTERN_DATE         'y'
#define RENAME_PATTERN_DISC_NUMBER  'd'
#define RENAME_PATTERN_DISC_COUNT   'e'
#define RENAME_PATTERN_FILENAME     'f'
#define RENAME_PATTERN_GENRE        'g'
#define RENAME_PATTERN_TITLE        't'
#define RENAME_PATTERN_TRACK_NUMBER 'n'
#define RENAME_PATTERN_TRACK_COUNT  'l'

/* default values (if a given tag does not exist, the folder 
 * hierarchy will be created using those values) 
 * TODO let the user sets those values (gconf)? */
#define RENAME_PATTERN_DEFAULT_ARTIST     N_("Unknown artist")
#define RENAME_PATTERN_DEFAULT_ALBUM      N_("Unknown album")
#define RENAME_PATTERN_DEFAULT_COMMENT    N_("No comments")
#define RENAME_PATTERN_DEFAULT_FILENAME   N_("Unknown filename")
#define RENAME_PATTERN_DEFAULT_GENRE      N_("Unknown")
#define RENAME_PATTERN_DEFAULT_TITLE      N_("Unknown title")
/* "numerical" values */
#define RENAME_PATTERN_DEFAULT_DISC_NUMBER  "1"
#define RENAME_PATTERN_DEFAULT_DATE         "0000"
#define RENAME_PATTERN_DEFAULT_TRACK_NUMBER "00"

#define LIBGNAC_RENAME_PATTERN_GET_ID(o) \
        (((LibgnacRenamePattern*)(o))->id)
#define LIBGNAC_RENAME_PATTERN_GET_PATTERN(o) \
        (g_regex_get_pattern(((LibgnacRenamePattern*)(o))->regex))
#define LIBGNAC_RENAME_PATTERN_GET_REGEX(o) \
        (((LibgnacRenamePattern*)(o))->regex)

typedef struct {
  const GRegex *regex;
  gchar        *replace;
  gchar         id; /* single letter representing the pattern */
} LibgnacRenamePattern;

typedef enum 
{
  FOLDER_CURRENT,
  FOLDER_SUBDIRECTORY,
  FOLDER_SELECTED,
  FOLDER_TYPE_N
} GnacFolderId;

void
libgnac_output_finalize(void);

void
libgnac_output_build_output(LibgnacConverterItem  *item, 
                            GError               **error);

gchar *
libgnac_output_get_preview_from_pattern(const gchar *pattern,
                                        gboolean     strip_special);

G_END_DECLS

#endif /* __LIBGNAC_OUTPUT_H__ */
