/*
 * $Id: gnac-bars.c 380 2009-07-27 12:07:50Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "gnac-bars.h"
#include "gnac-main.h"
#include "gnac-prefs.h"
#include "gnac-properties.h"
#include "gnac-stock-items.h"
#include "gnac-ui.h"

extern gint nb_files_total;

void
gnac_bars_on_row_inserted(void)
{
  gboolean   activate;
  GtkWidget *convert_button;
  GtkAction *action;

  activate = (nb_files_total != 0);

  action = gnac_ui_get_action("convert_item");
  gtk_action_set_sensitive(action, activate);

  convert_button = gnac_ui_get_widget("convert_button");
  gtk_widget_set_sensitive(convert_button, activate);

  gnac_bars_activate_clear(activate);
}


void
gnac_bars_on_row_deleted(void)
{
  GtkWidget *convert_button;
  GtkAction *action;

  action = gnac_ui_get_action("convert_item");
  gtk_action_set_sensitive(action, FALSE);

  convert_button = gnac_ui_get_widget("convert_button");

  gtk_widget_set_sensitive(convert_button, FALSE);

  gnac_bars_activate_remove(FALSE);
  gnac_bars_activate_clear(FALSE);
}

void 
gnac_bars_on_add_files(void)
{
  GtkWidget *convert_button;
  GtkAction *action;

  convert_button = gnac_ui_get_widget("convert_button");
  action = gnac_ui_get_action("convert_item");

  gtk_action_set_sensitive(action, TRUE);
  gtk_widget_set_sensitive(convert_button, TRUE);

  gtk_button_set_label(GTK_BUTTON(convert_button), GTK_STOCK_CANCEL);
  g_object_set(G_OBJECT(action), "stock-id", GTK_STOCK_CANCEL, NULL);

  gnac_ui_disable_profiles();

  gnac_bars_activate_add(FALSE);
  gnac_bars_activate_clear(FALSE);
  gnac_bars_activate_remove(FALSE);
}

void 
gnac_bars_on_add_files_finished(void)
{
  GtkWidget *convert_button;
  GtkAction *action;

  convert_button = gnac_ui_get_widget("convert_button");
  action = gnac_ui_get_action("convert_item");

  gtk_button_set_label(GTK_BUTTON(convert_button), GTK_STOCK_CONVERT);
  g_object_set(G_OBJECT(action), "stock-id", GTK_STOCK_CONVERT, NULL);

  gnac_ui_enable_profiles();

  gnac_bars_activate_add(TRUE);
  gnac_bars_activate_clear(TRUE);
  gnac_bars_activate_remove(TRUE);
}

void
gnac_bars_on_convert(void)
{
  GtkWidget *file_list;
  GtkWidget *convert_button;
  GtkAction *action;

  file_list = gnac_ui_get_widget("file_list");
  convert_button = gnac_ui_get_widget("convert_button");
  action = gnac_ui_get_action("convert_item");

  gtk_widget_set_sensitive(file_list, FALSE);

  gtk_button_set_label(GTK_BUTTON(convert_button), GTK_STOCK_STOP);
  g_object_set(G_OBJECT(action), "stock-id", GTK_STOCK_STOP, NULL);

  gnac_ui_disable_profiles();

  gnac_bars_activate_add(FALSE);
  gnac_bars_activate_clear(FALSE);
  gnac_bars_activate_remove(FALSE);
  gnac_bars_activate_pause(TRUE);
  gnac_bars_activate_properties(FALSE);
  gnac_bars_activate_preferences(FALSE);
}

void
gnac_bars_on_convert_stop(void)
{
  GtkWidget *file_list;
  GtkWidget *convert_button;
  GtkAction *action;

  file_list = gnac_ui_get_widget("file_list");
  convert_button = gnac_ui_get_widget("convert_button");
  action = gnac_ui_get_action("convert_item");

  gtk_widget_set_sensitive(file_list, TRUE);

  gtk_button_set_label(GTK_BUTTON(convert_button), GTK_STOCK_CONVERT);
  g_object_set(G_OBJECT(action), "stock-id", GTK_STOCK_CONVERT, NULL);

  gnac_ui_enable_profiles();

  gnac_bars_activate_add(TRUE);
  gnac_bars_activate_clear((nb_files_total != 0));
  gnac_bars_activate_pause(FALSE);
  gnac_bars_activate_preferences(TRUE);
}


void
gnac_bars_on_convert_pause(void)
{
  GtkAction *action;
  GtkWidget *button;

  action = gnac_ui_get_action("pause_item");
  g_object_set(G_OBJECT(action), "stock-id", GNAC_STOCK_RESUME, NULL);

  action = gnac_ui_get_action("tray_pause_item");
  g_object_set(G_OBJECT(action), "stock-id", GNAC_STOCK_RESUME, NULL);

  button = gnac_ui_get_widget("pause_button");
  gtk_button_set_label(GTK_BUTTON(button), GNAC_STOCK_RESUME);

  /* update the status bar */
  gnac_ui_append_status(_("paused"));
}


void
gnac_bars_on_convert_resume(void)
{
  GtkAction *action;
  GtkWidget *button;

  action = gnac_ui_get_action("pause_item");
  g_object_set(G_OBJECT(action), "stock-id", GTK_STOCK_MEDIA_PAUSE, NULL);

  action = gnac_ui_get_action("tray_pause_item");
  g_object_set(G_OBJECT(action), "stock-id", GTK_STOCK_MEDIA_PAUSE, NULL);

  button = gnac_ui_get_widget("pause_button");
  gtk_button_set_label(GTK_BUTTON(button), GTK_STOCK_MEDIA_PAUSE);
}


void
gnac_bars_activate_properties(gboolean activate)
{
  GtkAction *action;
  action = gnac_ui_get_action("properties_item");
  gtk_action_set_sensitive(action, activate);
}


void
gnac_bars_activate_pause(gboolean activate)
{
  GtkAction *action;
  GtkWidget *widget;
  
  action = gnac_ui_get_action("pause_item");
  gtk_action_set_visible(action, activate);

  widget = gnac_ui_get_widget("pause_button");
  gtk_widget_set_sensitive(widget, activate);
}


void
gnac_bars_activate_add(gboolean activate)
{
  GtkAction *action;
  action = gnac_ui_get_action("add_item");
  gtk_action_set_sensitive(action, activate);
}


void
gnac_bars_activate_remove(gboolean activate)
{
  GtkAction *action;
  action = gnac_ui_get_action("remove_item");
  gtk_action_set_sensitive(action, activate);
}
        

void
gnac_bars_activate_clear(gboolean activate)
{
  GtkAction *action;
  action = gnac_ui_get_action("clear_item");
  gtk_action_set_sensitive(action, activate);
}


void
gnac_bars_activate_preferences(gboolean activate)
{
  GtkAction *action;
  action = gnac_ui_get_action("prefs_item");
  gtk_action_set_sensitive(action, activate);
}
