/*
 * $Id: gnac-file-list.c 370 2009-07-14 17:13:07Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <gdk/gdkkeysyms.h>
#include <glib-object.h>

#include "gnac.h"
#include "gnac-bars.h"
#include "gnac-file-list.h"
#include "gnac-main.h"
#include "gnac-properties.h"
#include "gnac-ui.h"
#include "gnac-utils.h"

#include "libgnac-debug.h"


extern GnacState state;

static GtkWidget        *view = NULL;
static GtkTreeModel     *model = NULL;
static GtkTreeSelection *selection = NULL;

static GHashTable           *reference_table = NULL;

/* Tooltips configuration */
/*
static gboolean
gnac_file_list_tooltip_cb(GtkWidget  *widget,
			    gint        x,
			    gint        y,
			    gboolean    keyboard_tip,
			    GtkTooltip *tooltip,
			    gpointer    data)
{
  GtkTreeIter  iter;
  GtkTreePath *path = NULL;
  gchar       *tip_msg = NULL;


  if (!gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW(view), &x, &y,
					  keyboard_tip,
					  &model, &path, &iter)) {
    return FALSE;
  }

  gtk_tree_model_get(model, &iter, COL_TOOLTIP, &tip_msg, -1);
  
  if (!tip_msg) {
    return FALSE;
  }
  gtk_tooltip_set_text(tooltip, tip_msg);

  //gtk_tree_view_set_tooltip_cell (view, tooltip, path, column, cell)

  gtk_tree_path_free (path);

  //gtk_tree_view_set_tooltip_row (tree_view, tooltip, path);
  return TRUE;
}*/


static void
gnac_file_list_select_row(GtkTreeRowReference *reference)
{
  GtkTreePath *path;
  
  g_return_if_fail(reference);
  path = gtk_tree_row_reference_get_path(reference);
  
  gtk_tree_selection_unselect_all(selection);
  gtk_tree_selection_select_path(selection, path);
  gtk_tree_path_free(path);
}

static void
gnac_file_list_display_stock(GtkTreeViewColumn  *tree_column,
                             GtkCellRenderer    *cell,
                             GtkTreeModel       *tree_model,
                             GtkTreeIter        *iter,
                             gpointer            data)
{
  gchar *stock_id;

  gtk_tree_model_get(tree_model, iter, COL_STOCK, &stock_id, -1);
  g_object_set(G_OBJECT(cell), "stock-id", stock_id, NULL);
}

/*
 * Returns the number of rows that have been selected in the file list
 */
static gint
gnac_file_list_count_selected_rows(void)
{
  return gtk_tree_selection_count_selected_rows(selection);
}

gint
gnac_file_list_count_rows(void) 
{
  return g_hash_table_size(reference_table);
}


void
gnac_file_list_on_row_activated_cb(GtkTreeView       *treeview,
                                   GtkTreePath       *path,
                                   GtkTreeViewColumn *col,
                                   gpointer           user_data)
{
    gnac_properties_window_show();
}


static void
gnac_file_list_on_row_inserted_cb(GtkTreeModel *tree_model, 
                                  GtkTreePath  *path, 
                                  GtkTreeIter  *iter, 
                                  gpointer      user_data)
{
  gnac_properties_update_arrows();
}



static void
gnac_file_list_selection_changed_cb(GtkTreeSelection *selection, 
                                    gpointer          user_data)
{
  gint count;
  
  count = gtk_tree_selection_count_selected_rows(selection);
  
  if (count != 0 && state == GNAC_AUDIO_READY_STATE) {
    gnac_bars_activate_remove(TRUE);
  }

  if (count != 1) {
    gnac_bars_activate_properties(FALSE);
  } else {
    gnac_bars_activate_properties(TRUE);
  }
}


GtkWidget *
gnac_file_list_new(void)
{
  GtkCellRenderer   *renderer;
  GtkTreeViewColumn *column;

  /* Init reference table */
  reference_table = g_hash_table_new_full(g_str_hash, 
      g_str_equal,
      g_free,
      (GDestroyNotify)gtk_tree_row_reference_free);

  /* initialize the view */ 
  view = gnac_ui_get_widget("file_list");
  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(view), COL_TOOLTIP);

  /* This piece of code can be called if we want more complex tooltips
   * Note that we must adapt gnac_file_list_tooltip_cb
   *
   * g_object_set (view, "has-tooltip", TRUE, NULL);
   * g_signal_connect (view, "query-tooltip",
	 *	    G_CALLBACK (gnac_file_list_tooltip_cb), NULL);
   */

  /* initialize the model */
  
  model = GTK_TREE_MODEL(gtk_list_store_new(NUM_COLS, 
      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));

  renderer = gtk_cell_renderer_pixbuf_new();
  //g_object_set(G_OBJECT(renderer), "stock-size", 16, NULL);
  gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(view),
                                             -1,
                                             "Status",
                                             renderer,
                                             gnac_file_list_display_stock,
                                             NULL,
                                             NULL);

  renderer = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(renderer), "ypad", 5, NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                              -1,
                                              "Filelist",
                                              renderer,
                                              "text",
                                              COL_DISPLAY,
                                              NULL);

  gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
  
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(view),COL_URI);
  gtk_tree_view_column_set_sort_column_id (column,COL_URI);
  gtk_tree_view_column_clicked (column);


  column = gtk_tree_view_get_column(GTK_TREE_VIEW(view),COL_STOCK);
  gtk_tree_view_column_set_visible(column, FALSE);
  
  /* Connect signals */
  g_signal_connect(G_OBJECT(model), "row-inserted", 
      (GCallback) gnac_file_list_on_row_inserted_cb, NULL);

  /* configure selection */
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(selection), "changed", 
      (GCallback) gnac_file_list_selection_changed_cb, NULL);
  
  g_object_unref(model);

  return view;
}


GtkWidget *
gnac_file_list_get_view(void)
{
  return view;
}


GtkTreeModel *
gnac_file_list_get_model(void)
{
  return model;
}


GtkTreeSelection *
gnac_file_list_get_selection(void)
{
  return selection;
}

void
gnac_file_list_attach_default_model(gboolean attach)
{
  if (attach) {
    gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
    g_object_unref(G_OBJECT(model));
  } else {
    g_object_ref(G_OBJECT(model));
    gtk_tree_view_set_model(GTK_TREE_VIEW(view), NULL);
  }
}

void
gnac_file_list_add_row(const gchar *uri)
{
  GtkTreeRowReference *ref = NULL;
  GtkTreeIter iter;
  GtkTreePath *path = NULL;
  gchar *display;
  
  g_return_if_fail(uri);

  display = gnac_utils_get_display_name(uri, NULL); 

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                  COL_URI, uri,
                  COL_DISPLAY, display,
                  -1);

  g_free(display);

  path = gtk_tree_model_get_path(model, &iter);
  if (path) {
    ref = gtk_tree_row_reference_new(model, path);
    g_hash_table_insert(reference_table, g_strdup(uri), ref);
 
    gtk_tree_path_free(path);
  } else {
    libgnac_debug("Unable to create path");
  }
}


gboolean
gnac_file_list_get(GtkTreeRowReference  *reference,
                   gchar               **file_uri)
{
  g_return_val_if_fail(reference, FALSE);

  GtkTreeIter iter;
  GtkTreePath *path = NULL;

  path = gtk_tree_row_reference_get_path(reference);
  
  if (path) {
    gboolean ret = TRUE;
    if (gtk_tree_model_get_iter(model, &iter, path)) 
    {
      gtk_tree_model_get(model, &iter, 
                    COL_URI, file_uri, 
                  -1); 
    } 
    else { ret = FALSE; }
  
    gtk_tree_path_free(path);
    return ret;
  }

  return FALSE;
}


void 
gnac_file_list_remove_row(const gchar *uri)
{
  GtkTreeRowReference *ref = NULL;
  GtkTreePath   *path;

  ref = g_hash_table_lookup(reference_table, uri);
  if (ref)
  {
    path = gtk_tree_row_reference_get_path(ref);
    if (path) 
    {
        GtkTreeRowReference *new_ref = NULL;
        GtkTreeIter iter;

        if (gnac_file_list_has_next_row(ref)) {
          new_ref = gtk_tree_row_reference_copy(ref);
          gnac_file_list_get_next_row(&new_ref);
        } else if (gnac_file_list_has_prev_row(ref)) {
          new_ref = gtk_tree_row_reference_copy(ref);
          gnac_file_list_get_prev_row(&new_ref);
        }

        if (gtk_tree_model_get_iter(model, &iter, path)) 
        {
          gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
        }

        if (new_ref) {
          gnac_file_list_select_row(new_ref);
          gnac_properties_set_row(new_ref);
        }
    }
    g_hash_table_remove(reference_table, uri);

    gtk_tree_path_free(path);
  } 
  else { /* ref not defined */
    g_return_if_reached();
  } 
}


void
gnac_file_list_remove_all(void)
{
  gnac_file_list_attach_default_model(FALSE);
  gtk_list_store_clear(GTK_LIST_STORE(model));
  g_hash_table_remove_all(reference_table);
  gnac_file_list_attach_default_model(TRUE);
  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));

}


/*  
 *  Create a list of GtkTreeRowReference of all selected rows on the file list.
 *
 *  WARNING : To free the return value, use :
 *  g_list_foreach (list, (GFunc)gtk_tree_row_reference_free, NULL);
 *  g_list_free (list);
 */
GList *
gnac_file_list_get_selected_rows(void)
{
  GList *row_references = NULL ;
  GList *row_path;
  GList *next;

  row_path = gtk_tree_selection_get_selected_rows(selection, &model);
  
  /* Convert path to GtkTreeRowReference */
  next = row_path;
  while (next) {
    row_references = g_list_prepend(row_references, 
        gtk_tree_row_reference_new(model, next->data));
    next = next->next;
  }

  row_references = g_list_reverse(row_references);

  g_list_foreach(row_path, (GFunc)gtk_tree_path_free,NULL);
  g_list_free(row_path);

  return row_references;
}


/*
 * XXX this function is never used
 * Sets reference to the currently selected row. If multiples row are selected, 
 * sets reference to the first row of the selection.
 */
/*static gboolean
gnac_file_list_get_first_row(GtkTreeRowReference **reference)
{
  GtkTreePath *first_path;

  first_path = gtk_tree_path_new_first();
  if (*reference) {
    gtk_tree_row_reference_free(*reference);
  }

  *reference = gtk_tree_row_reference_new(model, first_path);

  gtk_tree_path_free(first_path);

  return gtk_tree_row_reference_valid(*reference);
}*/


gboolean
gnac_file_list_get_current_row(GtkTreeRowReference **reference)
{
  GList  *rows_path;

  if (*reference) {
    gtk_tree_row_reference_free(*reference);
    *reference = NULL;
  }

  rows_path = gtk_tree_selection_get_selected_rows(selection, &model);

  /* select the first selected row*/
  *reference = gtk_tree_row_reference_new(model, (GtkTreePath*)(rows_path->data));

  g_list_foreach(rows_path, (GFunc)gtk_tree_path_free, NULL);
  g_list_free(rows_path);

  return gtk_tree_row_reference_valid(*reference);
}


gboolean
gnac_file_list_has_next_row(GtkTreeRowReference *reference)
{
  GtkTreePath         *path;
  GtkTreeRowReference *temp;
  gboolean             result;

  g_return_val_if_fail(reference, FALSE);

  path = gtk_tree_row_reference_get_path(reference);
  if (path) {
    gtk_tree_path_next(path);
    temp = gtk_tree_row_reference_new(model, path);
    result = gtk_tree_row_reference_valid(temp);

    gtk_tree_row_reference_free(temp);
    gtk_tree_path_free(path);

  } else {
    return FALSE;
  }

  return result;
}


gboolean
gnac_file_list_has_prev_row(GtkTreeRowReference *reference)
{
  GtkTreePath *path;
  GtkTreePath *first;
  gboolean     result;

  g_return_val_if_fail(reference, FALSE);

  first = gtk_tree_path_new_first();
  path = gtk_tree_row_reference_get_path(reference);

  if (path) {
    if (gtk_tree_path_compare(path, first) == 0) {
     result = FALSE;

    } else {
     result = TRUE;
    }

  } else { /* path not defined */
    result = FALSE;
  }

  gtk_tree_path_free(path);
  gtk_tree_path_free(first);

  return result;
}


gboolean
gnac_file_list_get_next_row(GtkTreeRowReference **reference)
{
  GtkTreePath *path;

  g_return_val_if_fail(*reference, FALSE);

  path = gtk_tree_row_reference_get_path(*reference);
  gtk_tree_path_next(path);
  gtk_tree_row_reference_free(*reference);
  *reference = gtk_tree_row_reference_new(model, path);
  gtk_tree_path_free(path);

  return gtk_tree_row_reference_valid(*reference);
}


gboolean
gnac_file_list_get_prev_row(GtkTreeRowReference **reference)
{
  GtkTreePath *path;
  GtkTreePath *first;
  gboolean     result;

  g_return_val_if_fail(*reference, FALSE);

  first = gtk_tree_path_new_first();
  path = gtk_tree_row_reference_get_path(*reference);

  gtk_tree_row_reference_free(*reference);
  *reference = NULL;

  if (gtk_tree_path_compare(path, first) == 0) {
    result = FALSE;

  } else {
    gtk_tree_path_prev(path);
    *reference = gtk_tree_row_reference_new(model, path);
    result = TRUE;
  }

  gtk_tree_path_free(path);
  gtk_tree_path_free(first);

  return result;
}


void
gnac_file_list_select_row_and_follow(GtkTreeRowReference *reference)
{
  GtkTreePath *path;
  
  g_return_if_fail(reference);
  path = gtk_tree_row_reference_get_path(reference);
  
  gtk_tree_selection_unselect_all(selection);
  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(view), path, NULL, TRUE, 0.5, 0.0);
  gtk_tree_selection_select_path(selection, path);
  gtk_tree_path_free(path);
}

void
gnac_file_list_select_uri_and_follow(const gchar *uri)
{
  GtkTreeRowReference *ref = NULL;
  GtkTreePath   *path;

  ref = g_hash_table_lookup(reference_table, uri);
  if (ref)
  {
    path = gtk_tree_row_reference_get_path(ref);
    if (path) 
    {
      gtk_tree_selection_unselect_all(selection);
      gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(view), path, NULL, TRUE, 0.5, 0.0);
      gtk_tree_selection_select_path(selection, path);
      gtk_tree_path_free(path);
    } 
    else { /* ref not defined */
      g_return_if_reached();
    } 
  }
}


void
gnac_file_list_visual_error(const gchar *uri, 
                            const gchar *msg,
                            const gchar *stock_item)
{
  GtkTreeRowReference *ref = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  GtkTreeViewColumn *column; 

  column = gtk_tree_view_get_column(GTK_TREE_VIEW(view),COL_STOCK);

  ref =  g_hash_table_lookup(reference_table, uri);
  if (!ref)
  {
     g_return_if_reached();
  }

  path = gtk_tree_row_reference_get_path(ref);
  if (gtk_tree_model_get_iter(model, &iter, path))
  {
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
                   COL_STOCK, stock_item ? stock_item : GTK_STOCK_DIALOG_ERROR,
                   COL_TOOLTIP, g_strdup(msg), -1);
    gtk_tree_path_free(path);

    if (!gtk_tree_view_column_get_visible(column)) {
      gtk_tree_view_column_set_visible(column, TRUE);
    }
  }
}


void
gnac_file_list_hide_visual_bar()
{
  GtkTreeViewColumn *column; 

  column = gtk_tree_view_get_column(GTK_TREE_VIEW(view),COL_STOCK);
  gtk_tree_view_column_set_visible(column, FALSE);
}

void
gnac_file_list_remove_visual_error(const gchar *uri)
{
  GtkTreeRowReference *ref = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;

  ref =  g_hash_table_lookup(reference_table, uri);
  if (!ref)
  {
     g_return_if_reached();
  }

  path = gtk_tree_row_reference_get_path(ref);
  if (gtk_tree_model_get_iter(model, &iter, path))
  {
    gchar *stock = NULL;
    gchar *tooltip = NULL;
    gtk_tree_model_get(model, &iter,
            COL_STOCK, &stock,
            COL_TOOLTIP, &tooltip,
            -1);

    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
                       COL_STOCK, NULL,
                       COL_TOOLTIP, NULL, -1);
    g_free(stock);
    g_free(tooltip);
  }
  gtk_tree_path_free(path);
}


gboolean
gnac_file_list_on_button_pressed(GtkWidget      *treeview, 
                                 GdkEventButton *event, 
                                 gpointer        user_data)
{  
  GtkTreeRowReference *reference;

  if (event->type == GDK_BUTTON_PRESS && event->button == 3) {

    if (gnac_file_list_count_selected_rows() <= 1) {
      GtkTreePath *path;

      if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview),
            (gint) event->x, (gint) event->y,
            &path, NULL, NULL, NULL)) 
      {
        reference = gtk_tree_row_reference_new(model, path);
        
        gnac_file_list_select_row(reference);

        gtk_tree_row_reference_free(reference);
        gtk_tree_path_free(path);
      }
    }
    
    gnac_ui_show_popup_menu(treeview, event, user_data);
    return TRUE;
  }

  return FALSE;
}


gboolean
gnac_file_list_popup_menu(GtkWidget *treeview, 
                          gpointer   user_data)
{
  gnac_ui_show_popup_menu(treeview, NULL, user_data);

  return TRUE;
}


/*
void
gnac_file_list_delete_key_pressed(GtkWidget   *widget,
                                  GdkEventKey *key,
                                  gpointer     data)
{
  if (key->keyval == GDK_Delete) {
    gnac_on_ui_remove_cb(widget, data);
  } 
}
*/


void
gnac_file_list_update_cursor(void)
{
  GdkCursor *cursor = NULL;
  if (state == GNAC_AUDIO_FILE_ACTION_STATE ||
      state == GNAC_AUDIO_CONVERT_STATE)
  {
    cursor = gdk_cursor_new(GDK_WATCH);
  }
  gdk_window_set_cursor(view->window, cursor);
}


/* XXX this function is never used */
/*void  
gnac_file_list_display_uri(GtkTreeViewColumn  *tree_column,
                           GtkCellRenderer    *cell,
                           GtkTreeModel       *tree_model,
                           GtkTreeIter        *iter,
                           gpointer            data)
{
  GError *err = NULL;
  gchar  *uri = NULL;
  gchar  *name = NULL;

  gtk_tree_model_get(tree_model, iter, COL_URI, &uri, -1);

  name = (gchar *)gnac_utils_get_display_name(uri, &err);
  if (err != NULL) {
    g_clear_error(&err);
    return;
  }
  g_object_set(G_OBJECT(cell), "text", name, NULL);
}*/


void
gnac_file_list_destroy(void)
{

 // if (model)
  //  g_object_unref(model);

  if (reference_table ) {
    g_hash_table_unref(reference_table);
    reference_table = NULL;
  }
}
