/*
 * $Id: gnac-gconf.c 372 2009-07-15 06:48:37Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <gconf/gconf-client.h>
#include <glib/gi18n.h>
#include <stdlib.h>

#include "gnac.h"
#include "gnac-gconf.h"

static const gchar *gnac_gconf_paths[] = {
  "/apps/gnac/display/tray_icon",
  "/apps/gnac/conversion/clear_sources",
  "/apps/gnac/conversion/strip_special",
  "/apps/gnac/conversion/rename_pattern",
  "/apps/gnac/conversion/rename_pattern_pattern",
  "/apps/gnac/conversion/folder_hierarchy",
  "/apps/gnac/conversion/folder_hierarchy_pattern",
  "/apps/gnac/conversion/folder_type",
  "/apps/gnac/conversion/destination_directory",
  "/apps/gnac/conversion/last_used_profile",
};

GConfClient *gconf_client;        


void
gnac_gconf_init(void)
{
  gconf_client = gconf_client_get_default();

  if (gconf_client == NULL) {
    g_critical(_("Could not create GConf client.\n"));
    exit(EXIT_FAILURE);
  }

  /* Watch the directory */
  gconf_client_add_dir(gconf_client,
      GCONF_PATH,
			GCONF_CLIENT_PRELOAD_NONE,
      NULL);
}


void
gnac_gconf_destroy(void)
{
  g_object_unref(gconf_client);
}


gboolean
gnac_gconf_get_boolean(GnacGConfParam param)
{
  GError *err = NULL;
  gboolean val;

  val = gconf_client_get_bool(gconf_client, gnac_gconf_paths[param], &err);

  if (err != NULL)
  {
    g_printerr(_("Unable to read key: %s\n"), err->message);
    g_error_free(err);
  }

  return val;
}


void
gnac_gconf_set_boolean(GnacGConfParam param, gboolean value)
{
  GError *err = NULL;

  gconf_client_set_bool(gconf_client, gnac_gconf_paths[param], value, &err);

  if (err != NULL)
  {
    g_printerr(_("Unable to set key: %s\n"), err->message);
    g_error_free(err);
  }
}


gint
gnac_gconf_get_int(GnacGConfParam param)
{
  GError *err = NULL;
  gint val;

  val = gconf_client_get_int(gconf_client, gnac_gconf_paths[param], &err);

  if (err != NULL)
  {
    g_printerr(_("Unable to read key: %s\n"), err->message);
    g_error_free(err);
  }
  return val;
}


void
gnac_gconf_set_int(GnacGConfParam param, gint value)
{
  GError *err = NULL;

  gconf_client_set_int(gconf_client, gnac_gconf_paths[param], value, &err);
  if (err != NULL)
  {
    g_printerr(_("Unable to set key: %s\n"), err->message);
    g_error_free(err);
  }
}



gchar *
gnac_gconf_get_string(GnacGConfParam param)
{
  GError *err = NULL;
  gchar *val = NULL;

  val = gconf_client_get_string(gconf_client, gnac_gconf_paths[param], &err);

  if (err != NULL)
  {
    g_printerr(_("Unable to read key: %s\n"), err->message);
    g_error_free(err);
  }
  return val;
}


void
gnac_gconf_set_string(GnacGConfParam param, const gchar *value)
{
  GError *err = NULL;

  if (!value) return;

  gconf_client_set_string(gconf_client, gnac_gconf_paths[param], value, &err);
  if (err != NULL)
  {
    g_printerr(_("Unable to set key: %s\n"), err->message);
    g_error_free(err);
  }
}


/*
void
gconf_key_changed_cb(GConfClient* client,
                     guint cnxn_id,
                     GConfEntry* entry,
                     gpointer user_data)
{
  const char *key = NULL;
  GConfValue *value = NULL;

  if (entry != NULL)
  {
    key = gconf_entry_get_key(entry);
    value = gconf_entry_get_value(entry);

    g_printerr(_("Key changed: %s\n"), key);
    // TODO check what key it is and dispatch
  }
}*/
