/*
 * $Id: gnac-utils.c 366 2009-07-08 20:57:30Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <gio/gio.h>
#include <glib.h>

#include "gnac-utils.h"

#define GNAC_UTILS_ICON_BORDER_WIDTH 1

static guint64  average;
static guint64  total_samples;


gchar *
gnac_utils_get_display_name(const gchar  *uri, 
                            GError      **error)
{
  GError    *err = NULL;
  GFile     *file;
  GFileInfo *info;
  gchar *display_name;

  g_return_val_if_fail(!error || !*error, NULL);

  file = g_file_new_for_uri(uri);
  info = g_file_query_info(file, 
        G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, G_FILE_QUERY_INFO_NONE, 
        NULL, &err);
  if (err) {
    g_propagate_error(error, err);
    return NULL;
  }
  
  display_name = g_strdup(g_file_info_get_display_name(info));
  g_object_unref(info);
  g_object_unref(file);
  return display_name;
}


gchar *
gnac_utils_format_duration_for_display(guint64 duration)
{
  gint minutes;
  gint seconds;

  minutes = (gint) ((duration / 60) % 60);
  seconds = (gint) (duration % 60);

  if (duration >= 3600) 
  {
    return g_strdup_printf("%d:%02d:%02d", 
                    (gint) (duration / 3600), minutes, seconds);
  } 
  else
  {
    return g_strdup_printf("%d:%02d", minutes, seconds);
  } 

  return NULL;
}


void
gnac_utils_moving_avg_init(void)
{
  average = 0;
  total_samples = 0;
}


guint64 
gnac_utils_moving_avg_add_sample(guint64 sample)
{
  total_samples++;
  average = (average * (total_samples-1) + sample) / total_samples;

  return average;
}


void
gnac_utils_moving_avg_reset(void)
{
  gnac_utils_moving_avg_init();
}


/* FIXME not really efficient (although in our case we only 
 * have combos with less than 10 elements) */
gint
gnac_utils_get_combo_size(GtkComboBox *combo)
{
  gboolean      valid;
  guint         count = 0;
  GtkTreeIter   iter;
  GtkTreeModel *model;

  model = gtk_combo_box_get_model(combo);
  valid = gtk_tree_model_get_iter_first(model, &iter);

  while (valid)
  {
    count++;
    valid = gtk_tree_model_iter_next(model, &iter);
  }

  return count;
}


GdkPixbuf *
gnac_utils_scale_pixbuf(GdkPixbuf *pixbuf,
                        gint       new_width,
                        gint       new_height)
{
  gint height, width;
  gint n_height, n_width;

  g_return_val_if_fail(GDK_IS_PIXBUF(pixbuf), NULL);

  width = gdk_pixbuf_get_width(pixbuf);
  height = gdk_pixbuf_get_height(pixbuf);

  if (width > height)
  {
    n_width = new_width;
    n_height = n_width * height / width;
  }
  else
  {
    n_height = new_height;
    n_width = n_height * width / height;
  }

  return gdk_pixbuf_scale_simple(pixbuf, n_width, n_height, GDK_INTERP_BILINEAR);
}


GdkPixbuf *
gnac_utils_add_border_to_pixbuf(GdkPixbuf *pixbuf)
{
  GdkPixbuf *bordered;
  gint width, height;

  g_return_val_if_fail(GDK_IS_PIXBUF(pixbuf), NULL);

  width = gdk_pixbuf_get_width(pixbuf);
  height = gdk_pixbuf_get_height(pixbuf);
  bordered = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pixbuf),
                            gdk_pixbuf_get_has_alpha(pixbuf),
                            gdk_pixbuf_get_bits_per_sample(pixbuf),
                            width + (GNAC_UTILS_ICON_BORDER_WIDTH*2),
                            height +(GNAC_UTILS_ICON_BORDER_WIDTH*2));
  gdk_pixbuf_fill(bordered, 0xff);
  gdk_pixbuf_copy_area(pixbuf, 0, 0, width, height, bordered, 
            GNAC_UTILS_ICON_BORDER_WIDTH, GNAC_UTILS_ICON_BORDER_WIDTH);
  return bordered;
}
