/*
 * $Id: gnac-profiles-flac.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gnac-profiles-flac.h"

BasicFormatInfo flac_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-flac.xml",
  NULL,
  PKGDATADIR "/profiles/flac.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_flac_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_flac_init,
    gnac_profiles_flac_get_description,
    gnac_profiles_flac_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_flac_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_flac_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_flac_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_flac_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_flac_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_flac_load_specific_properties,
    gnac_profiles_flac_clean_up,
    NULL,
    gnac_profiles_flac_get_combo_format_name,
    gnac_profiles_flac_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_flac_init(UpdateTextBufferFunc call_back)
{
  update_text_buffer_call_back = call_back;

  gnac_profiles_default_init(&flac_bfi, 
    G_CALLBACK(gnac_profiles_flac_generate_pipeline));

  gnac_profiles_utils_all_in_one_for_slider(&flac_bfi, 
    G_CALLBACK(gnac_profiles_flac_generate_pipeline), "slider-compression",
    "//variable[@id='compression']/*");
  
  gnac_profiles_xml_engine_free_doc_xpath(flac_bfi.doc);
  flac_bfi.doc = NULL;

  return flac_bfi.format_id;
}


void
gnac_profiles_flac_generate_pipeline(GtkWidget *w)
{
  gchar *pipeline;

  pipeline = gnac_profiles_default_generate_pipeline(&flac_bfi);

  pipeline = gnac_profiles_utils_add_properties_slider(pipeline,&flac_bfi,
   "slider-compression", NULL); 
  
  update_text_buffer_call_back(pipeline);

  if (flac_bfi.pipeline != NULL) g_free(flac_bfi.pipeline);
  flac_bfi.pipeline = pipeline;
}

void
gnac_profiles_flac_fullfill_fields(AudioProfileFlac *profile)
{
  if (profile == NULL) {
    gnac_profiles_default_fullfill_fields(NULL, &flac_bfi);
    gnac_profiles_utils_reinit_values_slider(&flac_bfi,"slider-compression",
      NULL);
  } else {
    gnac_profiles_utils_set_values_slider(&flac_bfi,"slider-compression",
      profile->quality, NULL);
  }
}


gchar *
gnac_profiles_flac_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&flac_bfi);
}

void
gnac_profiles_flac_free_audio_profile(AudioProfileFlac *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 
  g_free(profile);
}

AudioProfileFlac *
gnac_profiles_flac_generate_audio_profile(GError **error)
{
  AudioProfileFlac *profile;
  AudioProfileGeneric *generic;

  generic = gnac_profiles_default_generate_audio_profile(&flac_bfi);
  profile = g_malloc(sizeof(AudioProfileFlac));
  profile->generic = generic;

  gnac_profiles_utils_get_values_slider_and_set(&flac_bfi,
    "slider-compression", &profile->quality,NULL);
  
  return profile;
}


GtkWidget *
gnac_profiles_flac_get_widget(AudioProfileFlac *profile)
{
  if (profile != NULL) {
    gnac_profiles_flac_fullfill_fields(profile);
  }
  return gnac_profiles_default_get_widget(profile,&flac_bfi);
}

void
gnac_profiles_flac_save_profile(AudioProfileFlac *profile)
{
  XMLDoc *doc;
  gchar *quality_str;
  
  quality_str = gnac_profiles_utils_gdouble_to_gchararray(profile->quality);
  doc = gnac_profiles_default_save_profile(profile->generic, &flac_bfi);
  gnac_profiles_utils_save_specific_properties_in_file(doc, 
    "quality", quality_str, NULL);
  
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
  g_free(quality_str);
}


AudioProfileFlac*
gnac_profiles_flac_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  AudioProfileFlac *profile;
  gchar *quality_str;
  
  profile = g_malloc(sizeof(AudioProfileFlac));
  profile->generic = generic;
  gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/format-specific/",
    "quality", &quality_str, NULL);
  
  if (quality_str != NULL) {
    profile->quality = gnac_profiles_utils_gchararray_to_gdouble(quality_str);
    g_free(quality_str);
  }
  
  return profile;
}

void
gnac_profiles_flac_clean_up(void)
{
  gnac_profiles_default_clean_up(&flac_bfi);
  gnac_profiles_utils_free_slider_values(&flac_bfi, "slider-compression",
    NULL);
  gnac_profiles_utils_free_basic_format_info_content(&flac_bfi);
}

const gchar *
gnac_profiles_flac_get_plugin_name(void)
{
  return flac_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_flac_get_description(void)
{
  return flac_bfi.description;
}
