/*
 * $Id: gnac-profiles-vorbis.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gnac-profiles-vorbis.h"

BasicFormatInfo vorbis_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-vorbis.xml",
  NULL,
  PKGDATADIR "/profiles/vorbis.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static void
gnac_profiles_vorbis_vbr_on_changed(GtkComboBox *widget);

static void
gnac_profiles_vorbis_advanced_bitrate_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data);

static AudioProfileVorbis *
gnac_profiles_vorbis_allocate_audio_profile(void);

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_vorbis_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_vorbis_init,
    gnac_profiles_vorbis_get_description,
    gnac_profiles_vorbis_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_vorbis_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_vorbis_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_vorbis_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_vorbis_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_vorbis_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_vorbis_load_specific_properties,
    gnac_profiles_vorbis_clean_up,
    NULL,
    gnac_profiles_vorbis_get_combo_format_name,
    gnac_profiles_vorbis_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_vorbis_init(UpdateTextBufferFunc call_back)
{
  GtkWidget *widget;

  update_text_buffer_call_back = call_back;

  gnac_profiles_default_init(&vorbis_bfi, 
    G_CALLBACK(gnac_profiles_vorbis_generate_pipeline));

  //Bitrate
  gnac_profiles_utils_all_in_one_for_combo(&vorbis_bfi, 
    G_CALLBACK(gnac_profiles_vorbis_generate_pipeline), "combo-bitrate",
    "//variable[@id='bitrate']/*");
  
  //Quality
  widget = gnac_profiles_utils_all_in_one_for_slider(&vorbis_bfi, 
    G_CALLBACK(gnac_profiles_vorbis_generate_pipeline), "slider-quality",
    "//variable[@id='quality']/*");
  gtk_scale_set_digits(GTK_SCALE(widget), 1);
 
  //Min bitrate
  widget = gnac_profiles_utils_all_in_one_for_combo(&vorbis_bfi, 
    G_CALLBACK(gnac_profiles_vorbis_generate_pipeline), "combo-min-vbr",
    "//variable[@id='vbr-min-bitrate']/*");
  gnac_profiles_utils_add_toggle_signal(&vorbis_bfi, "checkbutton-min-bitrate",
    G_CALLBACK(gnac_profiles_vorbis_advanced_bitrate_on_toggle), widget);
  
  //Max bitrate
  widget = gnac_profiles_utils_all_in_one_for_combo(&vorbis_bfi, 
    G_CALLBACK(gnac_profiles_vorbis_generate_pipeline), "combo-max-vbr",
    "//variable[@id='vbr-max-bitrate']/*");
  gnac_profiles_utils_add_toggle_signal(&vorbis_bfi, "checkbutton-max-bitrate",
    G_CALLBACK(gnac_profiles_vorbis_advanced_bitrate_on_toggle), widget);

  //Encoding mode
  widget = gnac_profiles_utils_all_in_one_for_combo(&vorbis_bfi, 
    G_CALLBACK(gnac_profiles_vorbis_generate_pipeline), "combo-encoding-mode",
    "//variable[@id='vbr']/*");
  
  gnac_profiles_vorbis_vbr_on_changed(GTK_COMBO_BOX(widget));
  
  gnac_profiles_xml_engine_free_doc_xpath(vorbis_bfi.doc);
  vorbis_bfi.doc = NULL;

  return vorbis_bfi.format_id;
}

static void
gnac_profiles_vorbis_vbr_on_changed(GtkComboBox *widget)
{
  gint index;
  gint i;
  
  GtkWidget *widgets_tab[8] = {
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "label-quality")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "hbox-quality")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "label-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "hbox-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "checkbutton-min-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "checkbutton-max-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "combo-max-vbr")),
    GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "combo-min-vbr"))
  };
  
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  for (i = 0; i<8; ++i) {
    if  (   ((index == 0) && ((i >= 4) || (i <= 1)))
        ||  ((index == 1) && ((i == 2) || (i == 3)))) {
      gtk_widget_show(widgets_tab[i]);
    } else {
      gtk_widget_hide(widgets_tab[i]);
    }
  }
}


void
gnac_profiles_vorbis_generate_pipeline(GtkWidget *w)
{
  GtkWidget *widget; 
  gchar *pipeline;
  gint index;

  pipeline = gnac_profiles_default_generate_pipeline(&vorbis_bfi);
  
  widget = GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "combo-encoding-mode"));

  if (w == widget) {
    gnac_profiles_vorbis_vbr_on_changed(GTK_COMBO_BOX(w));
  }

  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  if (index == 1) {
    pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &vorbis_bfi, 
      "combo-bitrate", NULL); 
  } else if (index == 0) {
    pipeline = gnac_profiles_utils_add_properties_slider_format(pipeline, &vorbis_bfi, 
      "%.1f", "slider-quality", NULL); 
    
    pipeline = gnac_profiles_utils_add_properties_checked_combo(pipeline, &vorbis_bfi, 
      "combo-min-vbr", "checkbutton-min-bitrate",
      "combo-max-vbr", "checkbutton-max-bitrate",
      NULL);
  }

  pipeline = gnac_profiles_utils_add_pipe(pipeline,vorbis_bfi.pipeline_multiplexers);
  update_text_buffer_call_back(pipeline);


  if (vorbis_bfi.pipeline != NULL) g_free(vorbis_bfi.pipeline);
  vorbis_bfi.pipeline = pipeline;

}

static void
gnac_profiles_vorbis_advanced_bitrate_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  gnac_profiles_utils_on_toggle_optionnal_property(togglebutton,user_data);
  gnac_profiles_vorbis_generate_pipeline(GTK_WIDGET(togglebutton));
}

void
gnac_profiles_vorbis_fullfill_fields(AudioProfileVorbis *profile)
{
  GtkWidget *widget;

  if (profile == NULL) {
    gnac_profiles_default_fullfill_fields(NULL, &vorbis_bfi);
    gnac_profiles_utils_reinit_values_combo(&vorbis_bfi,"combo-encoding-mode",
      "combo-bitrate","combo-min-vbr", "combo-max-vbr", NULL);
    gnac_profiles_utils_reinit_values_slider(&vorbis_bfi,"slider-quality", NULL);
   
    gnac_profiles_utils_set_active_toggle_button(&vorbis_bfi, FALSE,
      "checkbutton-min-bitrate", "checkbutton-max-bitrate", NULL);
      
  } else {
    widget = GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder, "combo-encoding-mode"));
    gnac_profiles_default_fullfill_fields(profile->generic,&vorbis_bfi);
    if (profile->quality == -1) {
      gnac_profiles_utils_set_values_combo(&vorbis_bfi, "combo-bitrate", 
        profile->bitrate, NULL);
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget),1);
  } else {
      gnac_profiles_utils_set_values_slider(&vorbis_bfi,"slider-quality",
        profile->quality, NULL);
      gnac_profiles_utils_set_values_checked_combo(&vorbis_bfi, 
        "combo-min-vbr", "checkbutton-min-bitrate", profile->min_bitrate,
        "combo-max-vbr", "checkbutton-max-bitrate", profile->max_bitrate, NULL);
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget),0);
    }
  }
}


gchar *
gnac_profiles_vorbis_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&vorbis_bfi);
}

static AudioProfileVorbis *
gnac_profiles_vorbis_allocate_audio_profile(void)
{
  AudioProfileVorbis *profile;

  profile = g_malloc(sizeof(AudioProfileVorbis));
  
  profile->quality = -1;
  profile->bitrate = NULL;
  profile->min_bitrate = NULL;
  profile->max_bitrate = NULL;

  return profile;
}

void
gnac_profiles_vorbis_free_audio_profile(AudioProfileVorbis *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 
  
  if (profile->bitrate != NULL) g_free(profile->bitrate);
  if (profile->min_bitrate != NULL) g_free(profile->min_bitrate);
  if (profile->max_bitrate != NULL) g_free(profile->max_bitrate);
  g_free(profile);
}


AudioProfileVorbis *
gnac_profiles_vorbis_generate_audio_profile(GError **error)
{
  AudioProfileGeneric *generic;
  AudioProfileVorbis *profile;
  GtkWidget *widget;
  gint index;

  generic = gnac_profiles_default_generate_audio_profile(&vorbis_bfi);

  profile = gnac_profiles_vorbis_allocate_audio_profile();
  profile->generic = generic;

  widget = GTK_WIDGET(gtk_builder_get_object(vorbis_bfi.builder,
    "combo-encoding-mode"));
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  if (index == 1) {
    gnac_profiles_utils_get_values_combo_and_set(&vorbis_bfi,
      "combo-bitrate", &profile->bitrate,NULL);
  } else {
    gnac_profiles_utils_get_values_slider_and_set(&vorbis_bfi,
      "slider-quality", &profile->quality,NULL);
    
    gnac_profiles_utils_get_values_checked_combo_and_set(&vorbis_bfi, 
      "combo-min-vbr", "checkbutton-min-bitrate", &profile->min_bitrate,
      "combo-max-vbr", "checkbutton-max-bitrate", &profile->max_bitrate, NULL);
  }
  
  return profile;
}


GtkWidget *
gnac_profiles_vorbis_get_widget(AudioProfileVorbis *profile)
{
  if (profile != NULL) {
    gnac_profiles_vorbis_fullfill_fields(profile);
  }
  return gnac_profiles_default_get_widget(profile,&vorbis_bfi);
}

void
gnac_profiles_vorbis_save_profile(AudioProfileVorbis *profile)
{
  XMLDoc *doc;
  gchar *quality_str;
  
  quality_str = gnac_profiles_utils_gdouble_to_gchararray_format(profile->quality,
    "%.1f");
  doc = gnac_profiles_default_save_profile(profile->generic, &vorbis_bfi);
  gnac_profiles_utils_save_specific_properties_in_file(doc, 
    "quality", quality_str,
    "bitrate", (profile->bitrate == NULL) ? "" : profile->bitrate,
    "min-bitrate", (profile->min_bitrate == NULL) ? "" : profile->min_bitrate,
    "max-bitrate", (profile->max_bitrate == NULL) ? "" : profile->max_bitrate,
    NULL
    ); 
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
  g_free(quality_str);
}


AudioProfileVorbis *
gnac_profiles_vorbis_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  AudioProfileVorbis *profile;
  gchar *quality_str;
  
  profile = gnac_profiles_vorbis_allocate_audio_profile();
  profile->generic = generic;
  gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/format-specific/",
    "quality", &quality_str,
    "bitrate", &profile->bitrate,
    "min-bitrate", &profile->min_bitrate,
    "max-bitrate", &profile->max_bitrate,
    NULL
    );

  if (quality_str != NULL) {
    profile->quality = gnac_profiles_utils_gchararray_to_gdouble(quality_str);
    g_free(quality_str);
  }

  return profile;
}

void
gnac_profiles_vorbis_clean_up(void) {
  gnac_profiles_default_clean_up(&vorbis_bfi);
  gnac_profiles_utils_free_combo_values(&vorbis_bfi, "combo-bitrate",
    "combo-min-vbr", "combo-max-vbr", "combo-encoding-mode", NULL);
  gnac_profiles_utils_free_slider_values(&vorbis_bfi, "slider-quality",
    NULL);
  gnac_profiles_utils_free_basic_format_info_content(&vorbis_bfi);
}

const gchar *
gnac_profiles_vorbis_get_plugin_name(void)
{
  return vorbis_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_vorbis_get_description(void)
{
  return vorbis_bfi.description;
}
