/*
 * $Id: gnac-profiles-wav.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gnac-profiles-wav.h"

BasicFormatInfo wav_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-wav.xml",
  NULL,
  PKGDATADIR "/profiles/wav.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_wav_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_wav_init,
    gnac_profiles_wav_get_description,
    gnac_profiles_wav_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_wav_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_wav_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_wav_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_wav_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_wav_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_wav_load_specific_properties,
    gnac_profiles_wav_clean_up,
    NULL,
    gnac_profiles_wav_get_combo_format_name,
    gnac_profiles_wav_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_wav_init(UpdateTextBufferFunc call_back)
{
  update_text_buffer_call_back = call_back;

  gnac_profiles_default_init(&wav_bfi, 
    G_CALLBACK(gnac_profiles_wav_generate_pipeline));
  
  gnac_profiles_xml_engine_free_doc_xpath(wav_bfi.doc);
  wav_bfi.doc = NULL;

  return wav_bfi.format_id;
}

void
gnac_profiles_wav_generate_pipeline(GtkWidget *w)
{
  gchar *pipeline;

  pipeline = gnac_profiles_default_generate_pipeline(&wav_bfi);
  
  update_text_buffer_call_back(pipeline);

  if (wav_bfi.pipeline != NULL) g_free(wav_bfi.pipeline);
  wav_bfi.pipeline = pipeline;
}

void
gnac_profiles_wav_fullfill_fields(AudioProfileGeneric *profile)
{
}


gchar *
gnac_profiles_wav_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&wav_bfi);
}

void
gnac_profiles_wav_free_audio_profile(AudioProfileGeneric *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 
}

AudioProfileGeneric *
gnac_profiles_wav_generate_audio_profile(GError **error)
{
  return gnac_profiles_default_generate_audio_profile(&wav_bfi);
}


GtkWidget *
gnac_profiles_wav_get_widget(AudioProfileGeneric *profile)
{
  return gnac_profiles_default_get_widget(profile,&wav_bfi);
}

void
gnac_profiles_wav_save_profile(AudioProfileGeneric *profile)
{
  XMLDoc *doc;
  
  doc = gnac_profiles_default_save_profile(profile, &wav_bfi);
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
}


AudioProfileGeneric *
gnac_profiles_wav_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  return generic;
}

void
gnac_profiles_wav_clean_up(void)
{
  gnac_profiles_default_clean_up(&wav_bfi);
  gnac_profiles_utils_free_basic_format_info_content(&wav_bfi);
}

const gchar *
gnac_profiles_wav_get_plugin_name(void)
{
  return wav_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_wav_get_description(void)
{
  return wav_bfi.description;
}
