/*
 * $Id: gnac-profiles-default.h 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef GNAC_PROFILES_DEFAULT_H
#define GNAC_PROFILES_DEFAULT_H

#include "gnac-profiles-utils.h"

typedef const gchar *(*FormatModuleInitFunc)(UpdateTextBufferFunc update_pipeline_text_buffer);
typedef void (*FormatModuleGeneratePipelineFunc) (GtkWidget*);
typedef void *(*FormatModuleGenerateAudioProfileFunc) (GError **);
typedef void (*FormatModuleFreeAudioProfileFunc) (void *);
typedef void (*FormatModuleFullfillFieldsFunc) (void *);
typedef gchar *(*FormatModuleGetComboFormatNameFunc) (void);
typedef GtkWidget *(*FormatModuleGetWidgetFunc) (void *);
typedef void (*FormatModuleSaveInFileFunc) (void *);
typedef void *(*FormatModuleLoadSpecificPropertiesFunc) (XMLDoc *doc, AudioProfileGeneric *generic);
typedef const gchar *(*FormatModuleGetPluginName) (void);
typedef const gchar *(*FormatModuleGetDescription) (void);
typedef void (*FormatModuleCleanUp) (void);

//Should be used only by unknown module
typedef void (*FormatModuleSetPipelineTextViewFunc) (GtkTextView*, GtkWidget *);

typedef struct {
  FormatModuleInitFunc init;
  FormatModuleGetDescription get_description;
  FormatModuleGeneratePipelineFunc generate_pipeline;
  FormatModuleGenerateAudioProfileFunc generate_audio_profile;
  FormatModuleFreeAudioProfileFunc free_audio_profile;
  FormatModuleFullfillFieldsFunc fullfill_fields;
  FormatModuleGetWidgetFunc get_widget;
  FormatModuleSaveInFileFunc save_profile_in_file;
  FormatModuleLoadSpecificPropertiesFunc load_specific_properties;
  FormatModuleCleanUp clean_up;
  FormatModuleSetPipelineTextViewFunc set_pipeline_func;
  FormatModuleGetComboFormatNameFunc get_combo_format_name;
  FormatModuleGetPluginName get_plugin_name;
} FormatModuleFuncs;

typedef FormatModuleFuncs (*FormatModuleGetFuncs) (void);

void
gnac_profiles_default_init(BasicFormatInfo *bfi, GCallback pipeline_callback);

GtkWidget *
gnac_profiles_default_get_widget(void *profile, BasicFormatInfo *bfi);

gchar *
gnac_profiles_default_get_combo_format_name(BasicFormatInfo *bfi);

gchar *
gnac_profiles_default_generate_pipeline(BasicFormatInfo *bfi);

void
gnac_profiles_default_fullfill_fields(AudioProfileGeneric *profile, BasicFormatInfo *bfi);

AudioProfileGeneric *
gnac_profiles_default_generate_audio_profile(BasicFormatInfo *bfi);

XMLDoc *
gnac_profiles_default_save_profile(AudioProfileGeneric *profile,
BasicFormatInfo *bfi);

XMLDoc *
gnac_profiles_default_load_generic_audio_profile(const gchar *filename, 
  AudioProfileGeneric **generic);

void
gnac_profiles_default_clean_up(BasicFormatInfo *bfi);

#endif /* GNAC_PROFILES_DEFAULT_H */
