/*
 * $Id: gnac-profiles-utils.h 239 2009-03-11 10:09:00Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef GNAC_PROFILES_UTILS_H
#define GNAC_PROFILES_UTILS_H

//Suppress useless unused warning on callback parameters
#define DISCARD_PARAMETER(p) (void)p

#include <gtk/gtk.h>
#include <stdarg.h>
#include "gnac-profiles-xml-engine.h"

typedef struct _audio_profile_generic
{
  struct _audio_profile_generic *generic;
  gchar *name;
  gchar *description;
  gchar *format_id;
  gchar *format_name;
  gchar *extension;
  gchar *pipeline;
  gchar *rate;
  gchar *channels;
  gpointer *user_data;
}
AudioProfileGeneric;

typedef struct
{
  const gchar *gtkbuilder_xml_file;
  GtkBuilder *builder;
  const gchar *data_xml_file;
  XMLDoc *doc;
  gchar *pipeline;
  gchar *pipeline_encoder;
  gchar *pipeline_multiplexers;
  gchar *format_id;
  gchar *format_plugin_name;
  gchar *format_name;
  gchar *file_extension;
  gchar *description;
}
BasicFormatInfo;

typedef void (*UpdateTextBufferFunc) (const gchar*);
typedef void (*StandardCallBack) (GtkWidget *widget, gpointer data);

void
gnac_profiles_utils_init_name_extension(XMLDoc *doc, const gchar *format, 
  gchar **name, gchar **extension);

gchar *
gnac_profiles_utils_get_combo_format_name(gchar *name, gchar *extension);

gchar *
gnac_profiles_utils_init_description(XMLDoc *doc, const gchar *format);


GtkWidget *
gnac_profiles_utils_all_in_one_for_combo(BasicFormatInfo *bfi,
  GCallback call_back, const gchar *combo_name, const gchar *xpath_query);

GtkWidget *
gnac_profiles_utils_all_in_one_for_slider(BasicFormatInfo *bfi,
  GCallback call_back, const gchar *slider_name, const gchar *xpath_query);

GtkWidget *
gnac_profiles_utils_all_in_one_for_check(BasicFormatInfo *bfi,
  GCallback call_back, const gchar *check_name, const gchar *xpath_query);

GtkWidget *
gnac_profiles_utils_add_toggle_signal(BasicFormatInfo *bfi, 
  const gchar *check, GCallback f, gpointer user_data);

gint
gnac_profiles_utils_register_combo(GtkWidget *widget, XMLDoc *doc, 
  const gchar *xpath_query, GCallback call_back, gpointer user_data);

gint
gnac_profiles_utils_register_slider(GtkWidget *widget, XMLDoc *doc, 
  const gchar *xpath_query, GCallback call_back, gpointer user_data);

gint
gnac_profiles_utils_register_check(GtkWidget *widget, XMLDoc *doc, 
  const gchar *xpath_query, GCallback call_back, gpointer user_data);

GtkWidget *
gnac_profiles_utils_register_advanced_channels(GtkBuilder *builder, 
  XMLDoc *doc, GCallback call_back, gpointer user_data);

GtkWidget *
gnac_profiles_utils_register_advanced_sample_rate(GtkBuilder *builder, 
  XMLDoc *doc, GCallback call_back, gpointer user_data);

void
gnac_profiles_utils_add_values_combo(GtkWidget *combo, ComboValues *values);

void
gnac_profiles_utils_reinit_values_combo(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_set_value_combo(GtkWidget *combo, const gchar *value);

void
gnac_profiles_utils_set_values_combo(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_set_values_checked_combo(BasicFormatInfo *bfi, ...);

gchar *
gnac_profiles_utils_get_value_combo(GtkWidget *combo);

void
gnac_profiles_utils_get_values_combo_and_set(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_get_values_checked_combo_and_set(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_add_values_slider(GtkWidget *slider, SliderValues *values);

void
gnac_profiles_utils_reinit_values_slider(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_set_values_slider(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_set_values_checked_slider(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_get_values_slider_and_set(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_get_values_checked_slider_and_set(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_reinit_values_check(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_set_values_check(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_get_values_check_and_set(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_add_description_tooltip(BasicFormatInfo *bfi,
  const gchar *xpath_query, ...);

void
gnac_profiles_utils_init_raw_audioconvert(XMLDoc *doc);

gchar *
gnac_profiles_utils_get_basepipeline(const gchar *channels, const gchar *rate);

gchar *
gnac_profiles_utils_add_pipe(gchar *pipeline, gchar *new_pipe);

gchar *
gnac_profiles_utils_add_pipes(gchar *pipeline, GList *values);

gchar *
gnac_profiles_utils_add_property_combo(gchar *pipeline, GtkWidget *combo);

gchar *
gnac_profiles_utils_add_properties_combo(gchar *pipeline, BasicFormatInfo *bfi, ...);

gchar *
gnac_profiles_utils_add_properties_checked_combo(gchar *pipeline, BasicFormatInfo *bfi, ...);

gchar *
gnac_profiles_utils_add_property_slider(gchar *pipeline, const gchar *format,
  gdouble factor, GtkWidget *slider);

gchar *
gnac_profiles_utils_add_properties_slider(gchar *pipeline, BasicFormatInfo *bfi, ...);

gchar *
gnac_profiles_utils_add_properties_slider_format(gchar *pipeline,
  BasicFormatInfo *bfi, const gchar *format, ...);

gchar *
gnac_profiles_utils_add_properties_checked_slider(gchar *pipeline, BasicFormatInfo *bfi, ...);

gchar *
gnac_profiles_utils_add_properties_checked_slider_format(gchar *pipeline,
  BasicFormatInfo *bfi, const gchar *format, ...);

gchar *
gnac_profiles_utils_add_property_check(gchar *pipeline, GtkWidget *check);

gchar *
gnac_profiles_utils_add_properties_check(gchar *pipeline, BasicFormatInfo *bfi,
 ...);

void
gnac_profiles_utils_set_active_toggle_button(BasicFormatInfo *bfi, gboolean active, ...);

void
gnac_profiles_utils_on_toggle_optionnal_property(GtkToggleButton *togglebutton,
  gpointer user_data);

void
gnac_profiles_utils_save_specific_properties_in_file(XMLDoc *doc, ...);

void
gnac_profiles_utils_load_saved_profile(XMLDoc *doc, const gchar *base_query, 
  ...);

AudioProfileGeneric *
gnac_profiles_utils_allocate_audio_profile_generic(void);

void
gnac_profiles_utils_free_audio_profile_generic(AudioProfileGeneric *profile);

void
gnac_profiles_utils_free_basic_format_info_content(BasicFormatInfo *bfi);

void
gnac_profiles_utils_free_combo_values(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_free_slider_values(BasicFormatInfo *bfi, ...);

void
gnac_profiles_utils_free_check_values(BasicFormatInfo *bfi, ...);

gchar *
gnac_profiles_utils_gdouble_to_gchararray(gdouble value);

gchar *
gnac_profiles_utils_gdouble_to_gchararray_format(gdouble value, const gchar *format);

gdouble
gnac_profiles_utils_gchararray_to_gdouble(const gchar * value);

void
gnac_profiles_utils_display_error_dialog(GtkWidget *dialog);


#endif /* GNAC_PROFILES_UTILS_H */
